@@ -1,12 +1,15 @@
 """Link budget command-line utility"""
+import argparse
 import json
 import logging
-import argparse
+import sys
+
+from dateutil.parser import isoparse
 
 from . import calc, constants, propagation, pointing, util
 from .antenna import Antenna
 
-__version__ = "0.1.8"
+__version__ = "0.1.9"
 
 
 def get_parser():
@@ -214,47 +217,88 @@ def get_parser():
         help="Transponder bandwidth in Hz, required if the PEB is provided.")
 
     pos_p = parser.add_argument_group(
-        'Satellite and Earth Station Position Information')
+        title='Earth Station Position Information',
+        description="Note longitudes are positive east of the prime meridian "
+        "and negative otherwise (west), while latitudes are positive above "
+        "the equator (north) and negative otherwise (south).")
+    pos_p.add_argument('--rx-long',
+                       type=float,
+                       help='Rx station\'s longitude in degrees.')
+    pos_p.add_argument('--rx-lat',
+                       type=float,
+                       help='Rx station\'s latitude in degrees.')
     pos_p.add_argument(
+        '--rx-height',
+        type=float,
+        default=0,
+        help='Rx station\'s height in meters above mean sea level.')
+
+    pos_s = parser.add_argument_group(
+        title='Satellite or Radar Object Position Information',
+        description="Note longitudes are positive east of the prime meridian "
+        "and negative otherwise (west), while latitudes are positive above "
+        "the equator (north) and negative otherwise (south).")
+    pos_s.add_argument(
         '--sat-long',
         type=float,
-        help='Satellite\'s longitude. Negative to the West and positive to '
-        'the East.')
-    pos_p.add_argument(
-        '--rx-long',
+        help='Longitude in degrees of the satellite or radar object.')
+    pos_s.add_argument(
+        '--sat-lat',
         type=float,
-        help='Rx station\'s longitude. Negative to the West and positive '
-        'to the East.')
-    pos_p.add_argument(
-        '--rx-lat',
+        default=0,
+        help='Latitude in degrees of the satellite or radar object.')
+    pos_s.add_argument(
+        '--sat-alt',
         type=float,
-        help='Rx station\'s latitude. Positive to the North and negative '
-        'to the South.')
-    pos_p.add_argument(
+        default=constants.GEOSYNC_ORBIT,
+        help='Satellite or radar object\'s altitude in meters above '
+        'the reference ellipsoid.')
+    pos_s.add_argument(
         '--slant-range',
         type=float,
         help='Slant path length in km between the Rx station and the '
         'satellite or reflector.')
+    pos_s.add_argument(
+        '--ref-ellipsoid',
+        choices=['WGS84', 'GRS80'],
+        default='WGS84',
+        help="Reference Earth ellipsoid to use in the computation.")
+    pos_s.add_argument(
+        '--obs-time',
+        type=isoparse,
+        help="Observation time for the satellite position\'s prediction. "
+        "Requires the \'--sat-tle-name\' option and must be given in "
+        "ISO 8601 format.")
+
+    tle_p = parser.add_argument_group(title='Satellite TLE Information')
+    tle_p.add_argument('--sat-tle-name',
+                       help="Satellite name on CelesTrak\'s TLE database.")
+    tle_p.add_argument(
+        '--sat-tle-group',
+        choices=pointing.CELESTRAK_GROUPS,
+        default=None,
+        help="CelesTrak satellite group to restrict the search. "
+        "When undefined, searches the entire database.")
+    tle_p.add_argument(
+        '--tle-save-dir',
+        default=pointing.get_default_tle_dataset_dir(),
+        help="Directory where downloaded TLE datasets should be saved.")
+    tle_p.add_argument('--tle-no-save',
+                       action='store_true',
+                       default=False,
+                       help="Do not save the downloaded TLE datasets locally. "
+                       "A new download will be required every time.")
 
     radar_p = parser.add_argument_group('Radar Options')
     radar_p.add_argument(
         '--radar',
         default=False,
         action='store_true',
-        help='Activate radar mode so that the link budget considers the '
-        'path loss to and back from object.')
-    radar_p.add_argument('--radar-alt',
-                         type=float,
-                         help='Altitude of the radar object')
+        help='Activate monostatic radar mode so that the link budget '
+        'considers the path loss to and back from object.')
     radar_p.add_argument('--radar-cross-section',
                          type=float,
                          help='Radar cross-section of the radar object.')
-    radar_p.add_argument(
-        '--radar-bistatic',
-        default=False,
-        action='store_true',
-        help='Bistatic radar scenario, i.e., radar transmitter and receiver '
-        'are not collocated.')
     return parser
 
 
@@ -287,34 +331,67 @@ def validate(parser, args):
             "Target link availability must be between 95 and 99.999 %%")
 
     if (args.radar):
-        if (args.slant_range is None and args.radar_alt is None):
-            parser.error("Argument --radar-alt is required in radar mode "
-                         "if the --slant-range argument is not provided")
         if (args.radar_cross_section is None):
             parser.error("Argument --radar-cross-section is required in radar "
                          "mode (--radar)")
 
-    # Mutual exclusion between "--slant-range" and the rx/sat positioning args
-    pos_args = [args.sat_long, args.rx_long, args.rx_lat]
-    pos_arg_labels = ['--sat-long', '--rx-long', '--rx-lat']
-    missing_pos_args = []
-    defined_pos_args = []
-    for arg, label in zip(pos_args, pos_arg_labels):
-        if (arg is None):
-            missing_pos_args.append(label)
-        else:
-            defined_pos_args.append(label)
-    if (args.slant_range is None and len(missing_pos_args) > 0):
+    # When the slant range is provided, none of the coordinates matter (neither
+    # the satellite's nor the Rx station's). That is because the look angles
+    # are not computed when the slant range is given. The slant range suffices
+    # for the link budget calculations in general, except if the atmospheric
+    # loss models are active, in which case the coordinates are required.
+    #
+    # Furthermore, when the satellite TLE name is given, the satellite position
+    # is obtained by prediction with the TLE orbital data, so only the Rx
+    # coordinates are required. However, the user may also choose to inform the
+    # satellite coordinates directly.
+    if args.slant_range is None:
+        if args.sat_tle_name is None:  # satellite position by coordinates
+            required_args = {
+                '--sat-long': args.sat_long,
+                '--rx-long': args.rx_long,
+                '--rx-lat': args.rx_lat
+            }
+            # NOTE the --sat-lat, --sat-alt, and --rx-height args have default
+            # values.
+            mutex_source = '--sat-long'
+            mutex_args = {
+                '--sat-tle-name': args.sat_tle_name,
+                '--obs-time': args.obs_time
+            }
+        else:  # satellite position via TLE prediction
+            required_args = {
+                '--rx-long': args.rx_long,
+                '--rx-lat': args.rx_lat
+            }
+            mutex_source = '--sat-tle-name'
+            mutex_args = {'--sat-long': args.sat_long}
+
+    else:  # slant range is given
+        required_args = {}
+        mutex_source = '--slant-range'
+        mutex_args = {
+            '--sat-tle-name': args.sat_tle_name,
+            '--sat-long': args.sat_long,
+            '--rx-long': args.rx_long,
+            '--rx-lat': args.rx_lat,
+            '--obs-time': args.obs_time
+        }
+
+    missing_args = [key for key, val in required_args.items() if val is None]
+    if missing_args:
         parser.error("the following arguments are required: {}".format(
-            ", ".join(missing_pos_args)))
-    elif (args.slant_range is not None and len(defined_pos_args) > 0):
-        parser.error("argument{} {}: not allowed with argument "
-                     "--slant-range".format(
-                         "s" if len(defined_pos_args) > 1 else "",
-                         ", ".join(defined_pos_args)))
+            ", ".join(missing_args)))
+    unexpected_args = [
+        key for key, val in mutex_args.items() if val is not None
+    ]
+    if unexpected_args:
+        parser.error("argument{} {}: not allowed with argument {}".format(
+            "s" if len(unexpected_args) > 1 else "",
+            ", ".join(unexpected_args), mutex_source))
 
     # The atmospheric loss models require the Rx/satellite coordinates.
-    if len(missing_pos_args) > 0 and args.atmospheric_loss is None:
+    if args.slant_range is not None and args.atmospheric_loss is None:
         logging.error(
             "Unable to model the atmospheric loss without the Rx and "
             "satellite coordinates")
@@ -368,12 +445,22 @@ def analyze(args, verbose=False):
 
     # -------- Look angles --------
     if (args.slant_range is None):
-        # Satellite altitude
-        sat_alt = constants.GEOSYNC_ORBIT if not args.radar else args.radar_alt
+        if args.sat_tle_name is not None:
+            try:
+                sat_long, sat_lat, sat_alt = pointing.get_sat_pos_by_tle(
+                    args.sat_tle_name, args.sat_tle_group, args.obs_time,
+                    args.tle_save_dir, args.tle_no_save)
+            except ValueError:
+                sys.exit(1)
+        else:
+            sat_long = args.sat_long
+            sat_lat = args.sat_lat
+            sat_alt = args.sat_alt
 
         # Look angles
         elevation, azimuth, slant_range_m = pointing.look_angles(
-            args.sat_long, args.rx_long, args.rx_lat, sat_alt)
+            sat_long, sat_lat, sat_alt, args.rx_long, args.rx_lat,
+            args.rx_height, args.ref_ellipsoid)
 
         # Polarization skew
         #
@@ -381,14 +468,19 @@ def analyze(args, verbose=False):
         # assumed equal to 45 degrees for circular polarization, according to
         # Recommendation ITU-R P.838-3.
         if (args.polarization == 'linear'):
-            pol_skew = pointing.polarization_angle(args.sat_long, args.rx_long,
-                                                   args.rx_lat)
+            pol_skew = pointing.polarization_angle(sat_long, sat_lat,
+                                                   args.rx_long, args.rx_lat)
         else:
             pol_skew = 45
     else:
         elevation = azimuth = pol_skew = None
         slant_range_m = args.slant_range * 1e3  # km to m
 
+    if (elevation is not None and elevation < 0):
+        el_label = "Object" if args.radar else "Satellite"
+        logging.error("{} below the horizon".format(el_label))
+        return
+
     # -------- Tx dish gain --------
     # Skipped if the EIRP is given directly.
     if (args.eirp is None):
@@ -442,9 +534,10 @@ def analyze(args, verbose=False):
     else:
         radar_obj_gain_db = None
     path_loss_db = calc.path_loss(slant_range_m, args.freq, args.radar,
-                                  radar_obj_gain_db, args.radar_bistatic)
-    # TODO support bistatic radar. Add distance from radar object to rx
-    # station.
+                                  radar_obj_gain_db)
+    # TODO support bistatic radar mode. The path_loss function already supports
+    # it. We just need to add --tx-lat/--tx-long options to separate the Tx and
+    # Rx locations for bistatic mode.
 
     # -------- Atmospheric loss --------
     #
